Package: bench
Title: High Precision Timing of R Expressions
Version: 1.1.4
Authors@R: c(
    person("Jim", "Hester", role = "aut"),
    person("Davis", "Vaughan", , "davis@posit.co", role = c("aut", "cre")),
    person("Drew", "Schmidt", role = "ctb",
           comment = "read_proc_file implementation"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Tools to accurately benchmark and analyze execution times for
    R expressions.
License: MIT + file LICENSE
URL: https://bench.r-lib.org/, https://github.com/r-lib/bench
BugReports: https://github.com/r-lib/bench/issues
Depends: R (>= 4.0.0)
Imports: glue (>= 1.8.0), methods, pillar (>= 1.10.1), profmem (>=
        0.6.0), rlang (>= 1.1.4), stats, tibble (>= 3.2.1), utils
Suggests: covr, dplyr, forcats, ggbeeswarm, ggplot2 (>= 3.5.1),
        ggridges, parallel, scales, testthat (>= 3.2.3), tidyr (>=
        1.3.1), vctrs (>= 0.6.5), withr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-01-16
Encoding: UTF-8
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2025-01-16 22:07:42 UTC; davis
Author: Jim Hester [aut],
  Davis Vaughan [aut, cre],
  Drew Schmidt [ctb] (read_proc_file implementation),
  Posit Software, PBC [cph, fnd]
Maintainer: Davis Vaughan <davis@posit.co>
Repository: CRAN
Date/Publication: 2025-01-16 22:40:07 UTC
