# Generated by roxygen2: do not edit by hand

S3method("[",bench_bytes)
S3method("[",bench_expr)
S3method("[",bench_mark)
S3method("[",bench_time)
S3method("[[",bench_bytes)
S3method("[[",bench_time)
S3method(Ops,bench_bytes)
S3method(Ops,bench_time)
S3method(Summary,bench_time)
S3method(as.character,bench_bytes)
S3method(as.character,bench_expr)
S3method(as.character,bench_time)
S3method(as_bench_bytes,bench_bytes)
S3method(as_bench_bytes,default)
S3method(as_bench_bytes,numeric)
S3method(as_bench_time,bench_time)
S3method(as_bench_time,default)
S3method(as_bench_time,numeric)
S3method(format,bench_bytes)
S3method(format,bench_expr)
S3method(format,bench_time)
S3method(max,bench_bytes)
S3method(max,bench_time)
S3method(mean,bench_time)
S3method(min,bench_bytes)
S3method(min,bench_time)
S3method(pillar_shaft,bench_bytes)
S3method(pillar_shaft,bench_expr)
S3method(pillar_shaft,bench_time)
S3method(plot,bench_mark)
S3method(print,bench_bytes)
S3method(print,bench_expr)
S3method(print,bench_time)
S3method(rbind,bench_mark)
S3method(sum,bench_bytes)
S3method(sum,bench_time)
S3method(summary,bench_mark)
S3method(type_sum,bench_bytes)
S3method(type_sum,bench_expr)
S3method(type_sum,bench_time)
export(as_bench_bytes)
export(as_bench_mark)
export(as_bench_time)
export(bench_bytes)
export(bench_bytes_trans)
export(bench_load_average)
export(bench_memory)
export(bench_process_memory)
export(bench_time)
export(bench_time_trans)
export(hires_time)
export(mark)
export(press)
export(scale_color_bench_expr)
export(scale_colour_bench_expr)
export(scale_x_bench_bytes)
export(scale_x_bench_expr)
export(scale_x_bench_time)
export(scale_y_bench_bytes)
export(scale_y_bench_expr)
export(scale_y_bench_time)
export(system_time)
export(workout)
export(workout_expressions)
importFrom(methods,setOldClass)
importFrom(pillar,pillar_shaft)
importFrom(pillar,type_sum)
useDynLib(bench, .registration = TRUE)
