% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load.R
\name{bench_load_average}
\alias{bench_load_average}
\title{Get system load averages}
\usage{
bench_load_average()
}
\description{
Uses OS system APIs to return the load average for the past 1, 5 and 15 minutes.
}
